
AllLieSubalgebras := function(L)
    local e, U, T, t, j, k, i, N;

    e := Filtered(Elements(L), x -> x <> Zero(L));
    U := Subalgebra(L, []);
    U!.ind := 0;
    T := [U];
    t := 1;

    while t <= Length(T) do
        U := T[t];
        t := t+1;
        if Dimension(U) < Dimension(L) then 
            j := U!.ind;
            for k in [j+1..Length(e)] do
                N := Subalgebra(L, Concatenation(Basis(U), [e[k]]));
                N!.ind := k;
                i := Position(T, N);
                if IsBool(i) then Add(T, N); fi;
            od;
        fi;
        Print("position ",t," of ",Length(T),"\n");
    od;
    return T;
end; 

MyBaseMat := function( mat )
    local j;
    if Length(mat) = 0 then return mat; fi;
    TriangulizeMat(mat);
    j := Position(mat, 0*mat[1]);
    if IsBool(j) then return mat; fi;
    return mat{[1..j]};
end;

Products := function( L, b, c )
    local u, r, j, k, w, v, l, U;
    u := List( c, x -> LinearCombination(b, x));
    r := [];
    for j in [1..Length(u)] do
        for k in [j+1..Length(u)] do
            w := u[j]*u[k];
            v := Coefficients(b, w);
            if v <> 0*v and not IsBool(SolutionMat(c,v)) then 
                return fail;
            fi;
            Add(r, w);
        od;
    od;
    U := Subalgebra(L, r);
    r := List(Basis(U), x -> Coefficients(b, x));
    if Length(r) = 0 or SumIntersectionMat(r,c)[2]<> [] then 
        return fail; 
    fi;
    return r;
end;

IsSubalgByBasis := function( L, b, c )
    local u, i, j, v, l;
    u := List( c, x -> LinearCombination(b,x));
    for i in [1..Length(u)] do
        for j in [i+1..Length(u)] do
            v := Coefficients(b, u[i]*u[j]);
            l := SolutionMat(c, v);
            if IsBool(l) then return false; fi;
        od;
    od;
    return true;
end;

IsCompCont := function( a, b, c )

    # check that a is a complement to b
    if Length(a) + Length(b) < Length(a[1]) then return false; fi;
    if Length(SumIntersectionMat(a,b)[2]) > 0 then return false; fi; 

    # check that c is contained in b
    return ForAll(c, x -> not IsBool(SolutionMat(b,x)));
end;
      
CheckCand := function(L, b, c )
    local u, v, i, j, l; 
    u := List(c[1], x -> LinearCombination(b, x));
    v := List(c[2], x -> LinearCombination(b, x));
    for i in [1..Length(u)] do
        for j in [1..Length(v)] do
            l := Coefficients(b, u[i]*v[j]);
            if IsBool(SolutionMat(c[1],l)) then 
                return false;
            fi;
        od;
    od;
    return true;
end;

FindGradingAlt := function(L)
    local d, b, s, t, i, p, j, cand;

    d := Dimension(L);
    b := BasisVectors(Basis(L));
    s := AllSubspaces(d, 2);
    s := Filtered(s, x -> Length(x)>0 and Length(x)<d);

    Print("determine subalgs \n");
    t := List(s, x -> IsSubalgByBasis( L, b, x ));
    
    Print("getting candidates for A1 \n");
    cand := []; # pairs of [a,b] with a*a < b and b*b < b
    for i in [1..Length(s)] do
        if t[i] = false then 
            Print("starting ",i,"\n");
            p := Products(L, b, s[i]);
            if p <> fail and Length(p) + Length(s[i]) = d then 
                Add(cand, [s[i], p]);
            elif p <> fail then 
                Print(" -- hard case \n");
                for j in [1..Length(s)] do
                    if t[j] = true then 
                        if IsCompCont(s[i], s[j], p) then 
                            Add(cand, [s[i],s[j]]);
                        fi;
                    fi;
                od;
            fi;
        fi;
    od;

    Print("checking candidates \n");
    # check that a*b < a
    for i in [1..Length(cand)] do
        if CheckCand(L, b, cand[i]) = false then 
            cand[i] := false; 
        else
            cand[i][1] := Subspace(L, 
                          List(cand[i][1], x -> LinearCombination(b,x)));
            cand[i][2] := Subalgebra(L, 
                          List(cand[i][2], x -> LinearCombination(b,x)));
        fi;
    od;
    return Filtered(cand,x -> x <> false);
                
end;

VecToLie := function(L, e);
    return LinearCombination(Basis(L), e);
end;

LieToVec := function(l)
    return l![1];
end;

DepthVecLie := function(l)
    return PositionNonZero(l![1]);
end;

TrivialIntersection := function(u, v)
    local a, b, s;
    a := List(u, x -> x![1]);
    b := List(v, x -> x![1]);
    s := SumIntersectionMat(a,b)[2];
    return Length(s) = 0;
end;

MyPositions := function(u)
    local a, n, j, r, i;
    a := List(u, x -> x![1]);
    n := Length(a);
    j := PositionNonZero(a[n]);
    r := [];
    for i in [j+1..Length(a[1])] do
        if ForAll( [1..n], x -> a[x][i] = 0*a[x][i] ) then 
            Add(r, i);
        fi;
    od;
    return r;
end;

SpacesWithProducts := function( L )
    local p, d, b, e, l, S, D, i, T, s, w, j, h, k, u, v;

    p := Characteristic(LeftActingDomain(L));
    d := Dimension(L);
    b := Basis(L);
    e := NormedVectors(GF(p)^d);
    l := List(e, x -> VecToLie(L,x));
    S := List(l, x -> [[x], []]);
    D := [];

    for i in [2..d] do
        Print("step ",i," has ",Length(S)," to do \n");
        T := [];
        for s in S do
            w := MyPositions(s[1]);
            for j in w do
                h := Filtered(l, x -> DepthVecLie(x)=j);
                for k in [1..Length(h)] do
                    v := Concatenation(StructuralCopy(s[1]), [h[k]]);
                    u := List(s[1], x -> x*h[k]);
                    u := Concatenation(s[2], u);
                    u := BasisVectors(Basis(Subalgebra(L,u)));
                    if TrivialIntersection(v,u) then
                        if Length(v) + Length(u) = d then 
                            Add(D, [v,u]);
                        else
                            Add(T, [v,u]);
                        fi;
                    fi;
                od;
            od;
        od;
        S := T;
    od;
    return D;
end;

CheckCandLie := function(cand)
    local u, v, c, i, j, a, l;
    u := cand[1];
    v := cand[2];
    c := List(u, x -> x![1]);
    for i in [1..Length(u)] do
        for j in [1..Length(v)] do
            a := u[i]*v[j];
            l := a![1];
            if IsBool(SolutionMat(c,l)) then return false; fi;
        od;
    od;
    return true;
end;

FindGrading := function(L)
    local cand, i;

    cand := SpacesWithProducts(L);
    for i in [1..Length(cand)] do
        if CheckCandLie(cand[i]) = false then 
            cand[i] := false; 
        fi;
    od;
    return Filtered(cand,x -> x <> false);
end;

RandomGrading := function(L)
    local a, b, U, V, t, n, x, y, N, m, M;

    a := Random(L);
    b := Random(L);
    U := Subspace(L, [a,b]);
    V := Subalgebra(L, []);
    t := false;

    while true do
        if Dimension(U) + Dimension(V) = Dimension(L) then 
            return [U,V];
        fi;

        # u * u < v 
        n := [];
        for x in Basis(U) do
            for y in Basis(U) do
                Add(n, x*y);
            od;
        od;
        N := Subalgebra(L, Concatenation(BasisVectors(Basis(V)), n));
        if TrivialIntersection(Basis(U), Basis(N)) = false then 
            return fail;
        fi;
     
        # u * v < u
        m := [];
        for x in Basis(N) do
            for y in Basis(U) do
                Add(m, x*y);
            od;
        od;
        M := Subspace(L, Concatenation(Basis(U), m));
        if TrivialIntersection(Basis(M), Basis(N)) = false then 
            return fail;
        fi;

        if Dimension(M) + Dimension(N) = Dimension(L) then 
            # final check
            for x in Basis(M) do
                for y in Basis(M) do
                    if not x*y in N then return fail; fi;
                od;
            od;
            for x in Basis(M) do
                for y in Basis(N) do
                    if not x*y in M then return fail; fi;
                od;
            od;
            return [M,N];
        fi;
        if Dimension(M) = Dimension(U) and Dimension(N) = Dimension(V) then 
            a := Random(L);
            U := Subspace(L, Concatenation(Basis(U), [a]));
        else
            U := M;
            V := N;
        fi;
    od;
end;

Trials := function(L, l)
    local r, i, a;
    r := [];
    for i in [1..l] do
        a := RandomGrading(L);
        if a <> fail then Add(r, a); fi;
    od;
    return r;
end;

