
LieNaturalModule := function(L)
    local l;
    l := List(GeneratorsOfAlgebra(L), x -> x![1]);
    return GModuleByMats(l, LeftActingDomain(L));
end;

LieAdjointModule := function(L)
    local B, b, m;
    B := Basis(L);
    b := BasisVectors(B);
    m := List(b, x -> TransposedMat(AdjointMatrix(B,x)));
    return GModuleByMats(m, LeftActingDomain(L));
end;

InstallMethod( IsSimpleLieAlgebra, true, [IsLieAlgebra], 0, 
	function(L)
    if Dimension(L) <= 1 then return true; fi;
    return SMTX.IsIrreducible(LieAdjointModule(L));
end );

IsCSimpleLieAlgebra := function(L)
    if Dimension(L) <= 1 then return false; fi;
    return SMTX.IsAbsolutelyIrreducible(LieAdjointModule(L));
end;

ChiefSeriesLieAlgebra := function(L)
    local M, U;
    M := LieAdjointModule(L);
    if SMTX.IsIrreducible(M) then return [L]; fi;
    U := SMTX.BasesCompositionSeries(M);
    U := Filtered(U, x -> Length(x)>0);
    return List(U, x -> Subalgebra(L, x*Basis(L), "basis" ));
end;

InstallGlobalFunction( IdealsOfLieAlgebra, function(L)
    local M, U;
    M := LieAdjointModule(L);
    if SMTX.IsIrreducible(M) then return [L]; fi;
    U := SMTX.BasesSubmodules(M);
    U := Filtered(U, x -> Length(x)>0);
    return List(U, x -> Subalgebra(L, x*Basis(L), "basis" ));
end );

IdealOfLieAlgebra := function(L)
    local M, b, I;
    if Dimension(L) <= 1 then return false; fi;
    M := LieAdjointModule(L);
    if SMTX.IsIrreducible(M) then return false; fi;
    b := SMTX.Subbasis(M);
    I := Subalgebra( L, b * Basis(L), "basis" );
    SetIsIdealInParent(I, true);
    return I;
end;

InstallGlobalFunction( SimpleFactorsLieAlgebra, function(L)
    local simpl, queue, I, J, F;
    simpl := [];
    queue := [L];
    while Length(queue) > 0 do

        # take ideal
        I := queue[Length(queue)];
        Unbind(queue[Length(queue)]);

        # split it
        J := IdealOfLieAlgebra(I);

        # check
        if IsBool(J) then
            Add(simpl, I);
        else
            F := I/J;
            Add( queue, LieAlgebraByTable(LieTableByBasis(J, Basis(J))));
            Add( queue, LieAlgebraByTable(LieTableByBasis(F, Basis(F))));
        fi;
    od;
    return Filtered( simpl, x -> Dimension(x)>1 );
end );


