
PolysByDegreeSS := function( n, F )
    local q, f, c, p;
    q := Size(F);
    f := FamilyObj(Zero(F));
    c := List([1..n], x->AllIrreducibleMonicPolynomialCoeffsOfDegree(x,q));
    p := List(c, x->List(x, y->LaurentPolynomialByCoefficients(f,y,0)));
    return p;
end;

PolysByDegree := function( n, F )
    local p, m, j, k, l;
    p := PolysByDegreeSS(n,F);
    m := List([1..n], x -> []);
    for j in [1..n] do
        for k in DivisorsInt(j) do
            l := j/k;
            Append(m[j], List(p[k], x -> x^l));
        od;
    od;
    return m;
end;

PolysForPartition := function( pol, par )
    local com, pos, i, new, nps, f, l, j, c, k, s;

    # the first coordinate
    com := List( pol[par[1]], x -> [x] );
    pos := [1..Length(pol[par[1]])];

    # the later coordinates
    for i in [2..Length(par)] do

        # set up for next level
        new := [];
        nps := [];

        # set up new polys
        f := pol[par[i]];
        l := Length(f);

        # loop over current combinations
        for j in [1..Length(com)] do
            c := com[j];
            k := pos[j];

            # distinguish cases
            if par[i] <> par[i-1] then s := 1; else s := k; fi;

            # fill up
            Append( new, List( f{[s..l]}, x -> Concatenation(c,[x])));
            Append( nps, [s..l]);
        od;
        com := new;
        pos := nps;
    od;
    return com;
end;

JNFByPolys := function( pols, F )
    local b, n, M, c, i, j, k;
    b := List(pols, CompanionMat);
    n := Sum(List(b, Length));
    M := NullMat( n,n, F );
    c := 0;
    for i in [1..Length(b)] do
        for j in [1..Length(b[i])] do
            for k in [1..Length(b[i])] do
                M[c+j][c+k] := b[i][j][k];
            od;
        od;
        c := c + Length(b[i]);
    od;
    return M;
end;

InstallGlobalFunction( JordanNormalForms, function( arg )
    local n, F, pol, par, jnf, pi, pls, new;

    # catch arguments
    n := arg[1];
    F := arg[2];
    if IsBound(arg[3]) and arg[3] = "ss" then 
        pol := PolysByDegreeSS(n,F);
    else
        pol := PolysByDegree(n, F);
    fi;

    # set up
    par := Partitions(n);
    jnf := [];

    # loop over partitions
    for pi in par do

        # determine polynomials for pi
        pls := PolysForPartition( pol, pi );

        # compute JNFs
        new := List( pls, x -> JNFByPolys(x,F) );
 
        # store
        Append(jnf, new );
    od; 

    # that's it
    return jnf;
end );

SpinnUpCyclic := function( mat, v )
    local b, w, t;
    b := [v];
    repeat
        w := b[Length(b)]*mat;
        t := IsBool(SolutionMat(b,w));
        if t then 
            Add(b,w); 
        else
            return b;
        fi;
    until false;
end;

JordanTypeCyclic := function( F, mat, g, e )
    local I, d, t, n, l, i, v, b, c, m;

    # check
    if Length(mat)=0 then Error("this should not happen"); fi;

    # a special case
    if mat = 0*mat then 
        return Concatenation([Indeterminate(F)], List(mat, x -> 1));
    fi;
 
    # set up
    d := Degree(g);
    t := [g];
    m := ShallowCopy(mat);
    n := Length(m);
    I := IdentityMat(n, F);

    repeat 

        # get new exponent
        l := Value(g^(e-1),m); 
        while l = 0*l do
            e := e-1;
            l := Value(g^(e-1),m); 
        od;

        # extend t
        Add( t, e );

        # check
        if d*e = n then return t; fi;

        # set up for iteration
        i := First([1..Length(l)], x -> l[x] <> 0*l[x]);
        b := SpinnUpCyclic( m, I[i] );
        c := BaseSteinitzVectors(I, b).factorspace; 
        m := InducedActionFactor( [m], c, b )[1];
        n := Length(m);
        I := IdentityMat(n, F);
    until false;
end;

InstallGlobalFunction( JordanType, function(F, mat)
    local f, g, t, i, h, m, b, c, l, I;

    # set up
    f := CharacteristicPolynomial(mat);
    g := Collected(Factors(f));

    # a special case
    if Length(g) = 1 then 
        return [JordanTypeCyclic( F, mat, g[1][1], g[1][2] )];
    fi;

    # set up for loop
    t := [];
    I := IdentityMat(Length(mat),F);

    # loop over factors
    for i in [1..Length(g)] do
        h := g[i][1]^g[i][2];
        m := Value(h, mat);
        b := TriangulizedBasis( m );
        c := BaseSteinitzVectors(I, b).factorspace; 
        l := InducedActionFactor( [mat], c, b )[1];
        Add( t, JordanTypeCyclic( F, l, g[i][1], g[i][2] ) );
    od;
    return t;
end );


