
##
## check a subalgebra against the library
##
LIMIT := Length(SLTB2);

LieDBByInvs := function( d, e )
    return Position( SLIB[d], e );
end;

LieCheckDB := function( M, r )
    local d, e, j, L, t;

    # set up
    d := Dimension(M);
    e := LieRITInvariants(M);
    j := LieDBByInvs(d, e);
    if j = fail then return fail; fi;

    # get Lie algebra
    L := CheckOutDB(d, j);
    Print("try to match to ",[d,j],"\n");

    if IsBool(r) then
        t := LieIsomorphismTest( L, M );
        return [t, [d,j]];
    else
        t := LieRandomIsomorphismTest( L, M, r );
        return [t, [d,j]];
    fi;
end;

LieCheckDBFactors := function( U, r )
    local C, c, l, new; 
    new := [];
    C := SimpleFactorsLieAlgebra(U);
    for c in C do 
        if Dimension(c) < LIMIT then 
            l := LieCheckDB(c, r);
            if l = fail or l[1] <> true then 
                Add( new, c );
            fi;
        fi;
    od;
    return new;
end;


## 
## a crude approach
##
RandomSearch := function(L, t, r)
    local i, U, c;
    for i in [1..t] do
        U := Subalgebra(L, [Random(L),Random(L)]); 
        c := LieCheckDBFactors(U, r);
        if Length(c) > 0 then Error("found"); fi;
    od;
end;

RandomSearchSL := function(n, F, t, r)
    local L, J, j, i, U, c;
    L := MatrixLieAlgebra(F, n);
    J := JordanNormalForms(n, F);
    for j in J do
        for i in [1..t] do
            U := Subalgebra(L, [j,Random(L)]); 
            c := LieCheckDBFactors(U, r);
            if Length(c) > 0 then Error("found"); fi;
        od;
    od;
end;

