RSSInvariants := function( L )
local D, n, s, a;
	
    D := Derivations(Basis(L));
    n := NilElementsInDer(D);
    s := Filtered(n, x -> IsSandwichDer(L,x));
    a := List(s, x -> AutoBySandwich(LeftActingDomain(L), x));

	# restricted, der, ann, exp
	return [IsRestrictedLieAlgebra(L), Dimension(D), Size(s), Size(Group(a))];
end;


PrintRSSInvariants := function( L )
	Print("Simple algebra has invariants: res=", L!.res, " der=", L!.der, " ann=", L!.ann, " exp=", L!.exp, ".\n");
end;


SLIMIT := 20;
new := List([1..SLIMIT], x -> []);
possible := List([1..SLIMIT], x -> []);

RandomSearchSimple := function( L )
    local U, M, r, S, ischeck, s, dimS, isnew, ispossible, invs, K;

	M := [2, 2, 2, 2, 2, 3];	

    while true do
		r := Random(M);
		if r = 2 then
        	U := Subalgebra(L, [Random(L), Random(L)]); 
		else
			U := Subalgebra(L, [Random(L), Random(L), Random(L)]);
		fi;

        S := SimpleFactorsLieAlgebra(U);

		ischeck := false;

		for s in S do
			dimS := Dimension(s);

			if dimS <= SLIMIT then
				ischeck := true;
				Print("Checking simple constituent of dimension ", dimS, ".\n");
				isnew := true;
				ispossible := true;
				invs := RSSInvariants(s);
				s!.res := invs[1];
				s!.der := invs[2];
				s!.ann := invs[3];
				s!.exp := invs[4];

				Print("Simple constituent has invariants: res=", invs[1], " der=", invs[2], " ann=", invs[3], " exp=", invs[4], ".\n");
				
				# first check invariants
				Print("Checking against ", Size(Concatenation( SLDB2[dimS], new[dimS], possible[dimS] )), " algebras in the DB.\n");

				for K in Concatenation( SLDB2[dimS], new[dimS], possible[dimS] ) do
					if invs = [K!.res, K!.der, K!.ann, K!.exp] then
						Print("Found algebra with same invariants, initiate random iso test.\n");
						isnew := false;
						AddGensAndType(s);
						if LieRandomIsomorphismTest(s, K, 1000) = true then
							Print("Found isomorphic algebra.\n");
							ispossible := false;
							break;
						fi;
					fi;	
				od;

				if isnew then
					Add(new[dimS], s);
				fi;

				if ispossible and not isnew then
					Add(possible[dimS], s);
				fi;
			fi;
		od;

		if ischeck = true then
			Print("found ", Size(Flat(new)), " Lie algebras and ", Size(Flat(possible)), " candidates for further investigation.\n");
		fi;
    od;
end;






KaplanskySub := function( n )
local K, S, s, g1, g2;

	K := SimpleLieAlgebraAlternateMats(n);
	Print("Consider Kap_1(", n, ") of dimension ", Dimension(K), "\n");

	while true do
		g1 := Random(K);
		g2 := Random(K);
		S := Subalgebra(K, [g1, g2]);
		S := SimpleFactorsLieAlgebra(S);

		for s in S do
			if Dimension(s) < Dimension(K) then		
				Print("Found simple constituent of dimension ", Dimension(s), "\n");
			fi;
			#if Dimension(s) = Dimension(K)/2 then	
			#	Print("Found simple Lie algebra of index 2!\n");
			#	return [K, g1, g2, S, s];	
			#fi;
		od;
	od;
end;


simple := [];

KaplanskySimpleSub := function( n )
local K, S, g1, g2;

	K := SimpleLieAlgebraByGramMatrix1(n);
	Print("Consider Kap_1(", n, ") of dimension ", Dimension(K), "\n");

	while true do
		g1 := Random(K);
		g2 := Random(K);
		S := Subalgebra(K, [g1, g2]);
		
		if IsSimpleLieAlgebra(S) and Dimension(S) < Dimension(K) then
			Print("Found simple subalgebra of dimension ", Dimension(S), "\n");
			Add(simple, S);
		fi;
	od;
end;
