    
BasisByGens := function( L, gens )
    local F, B, d, base, coef, wrds, i, j, n, c;

    # set up
    F := LeftActingDomain(L);
    B := Basis(L);
    d := Length(B);

    # set up basis
    base := ShallowCopy(gens*B);
    coef := ShallowCopy(gens);
    wrds := List([1..Length(gens)],x -> [x]);
    if RankMat(coef) < Length(coef) then Error("redundant generators"); fi;

    # enlarge gens to base
    while Length(base) < d do
        for i in [1..Length(gens)] do
            for j in [1..Length(base)] do
                n := base[j]*base[i];
                c := Coefficients(B, n);
                if IsBool( SolutionMat(coef, c) ) then
                    Add( base, n );
                    Add( coef, c );
                    Add( wrds, Concatenation( [wrds[j], [i]]));
                fi;
            od;
        od;
    od;

    return rec( base := base, coef := coef, wrds := wrds );
end;

PresentationByGens := function(L, gens)
    local F, B, d, b, i, j, r, c, n, k, rels;

    # set up
    F := LeftActingDomain(L);
    B := Basis(L);
    d := Length(B);

    # enlarge gens to basis
    b := BasisByGens( L, gens );

    # compute relators 
    rels := [];
    for i in [1..d] do
        for j in [i+1..d] do

            # compute base[i] * base[j] = sum c[k] base[k]
            r := b.base[i] * b.base[j];
            c := SolutionMat( b.coef, Coefficients(B, r) );

            # determine related word
            n := [[b.wrds[i], b.wrds[j]], One(F)];
            for k in [1..Length(c)] do
                if c[k] <> 0*c[k] then Append(n, [b.wrds[k], -One(F)]); fi;
            od;
            Add( rels, n );
        od;
    od;

    return rels;
end;

AddPresentationOfLieAlg := function(L)
    if IsBound(L!.rels) then return; fi;
    L!.rels := PresentationByGens(L, L!.gens);
end;


