
StringList := function( m )
   local s, i;
   s := "[";
   for i in [1..Length(m)-1] do
       s := Concatenation(s,String(m[i]),",");
   od;
   return Concatenation(s,String(m[Length(m)]),"]");
end;

ModularSimpleLieAlgebra := function( F, type, m )
    local p, L;

    p := Characteristic(F);

    # type W 
    if type = "W" then
        if p=2 and m=[1] then return fail; fi;
        L := SimpleLieAlgebra( "W", m, F);
        if p=2 and Length(m) = 1 then L := LieDerivedSubalgebra(L); fi;
    fi;

    # type S 
    if type = "S" then
        if Length(m) = 1 then return fail; fi;
        L := SimpleLieAlgebra( "S", m, F );
        if Length(m) = 2 then L := LieDerivedSubalgebra(L); fi;
    fi;

    # type H
    if type = "H" then
        if not IsInt(Length(m)/2) or Length(m) < 2 then return fail; fi;
        L := SimpleLieAlgebra( "H", m, F );
    fi;

    if type = "K" then 
        if not IsInt((Length(m)-1)/2) or Length(m) < 3 then return fail; fi;
        L := SimpleLieAlgebra( "K", m, F );
    fi;

    SetName( L, Concatenation( type, StringList(m) ) );
    return L;
end;

ModularSimpleLieAlgebras := function( F, limit )
    local p, par, all, m, d, L, sub, try, A, B, C, i;

    p := Characteristic(F);
    par := Concatenation( List( [1..LogInt(2*limit+2,p)+1], Partitions ) );
    all := [];

    # type W 
    for m in par do 
        if p=2 and m=[1] then 
           d := limit+1;
        elif p=2 and Length(m)=1 then 
           d := Length(m)*p^Sum(m)-1;
        else
           d := Length(m)*p^Sum(m);
        fi;
        if d <= limit then 
            L := ModularSimpleLieAlgebra(F,"W",m);
            if Dimension(L) <> d then Error("dimension wrong for W"); fi;
            Add(all, L);
        fi;
    od;
Print("W done \n");

    # type S
    for m in par do
        if Length(m)>2 then
            d := (Length(m)-1)*(p^Sum(m)-1); 
        elif Length(m)=2 and (p>2 or not 1 in m) then 
            d := p^Sum(m)-2;
        else
            d := limit+1;
        fi;
        if d <= limit then
            L := ModularSimpleLieAlgebra(F,"S",m);
            if Dimension(L) <> d then Error("dimension wrong for S"); fi;
            Add(all, L);
        fi;
    od;
Print("S done \n");

    sub := Concatenation( List( [1..LogInt(limit+2,p)+1], Partitions ) );
    par := [];
    for i in [1..LogInt(limit+2,p)+1] do
        try := Filtered(sub, x -> Length(x) = i);
        for A in try do
            for B in try do
                C := Concatenation(A,B);
                AddSet(par, C);
            od;
        od;
    od;

    # type H
    for m in par do
        if p>2 or Length(m)>2 then 
            d := p^Sum(m)-2;
        else
            d := limit + 1;
        fi;
        if d <= limit then
            L := ModularSimpleLieAlgebra(F,"H",m);
            if Dimension(L) <> d then Error("dimension wrong for H"); fi;
            Add(all, L);
        fi;
    od;
Print("H done \n");

    if p = 2 then
        Sort( all, function(a,b) return Dimension(a)<Dimension(b); end );
        return all;
    fi;

    par := Concatenation(List([1..LogInt(limit,p)+1], OrderedPartitions));
    par := Filtered(par, x -> Length(x)>2);
    par := Filtered(par, x -> IsInt((Length(x)-1)/2));
            
    # type K 
    for m in par do
        if (Length(m)+3) mod p = 0 then 
            d := p^Sum(m)-1;
        else
            d := p^Sum(m);
        fi;
        if d <= limit then
            L := ModularSimpleLieAlgebra(F,"K",m);
            if Dimension(L) <> d then Error("dimension wrong for K"); fi;
            Add(all, L);
        fi;
    od;

    Sort( all, function(a,b) return Dimension(a)<Dimension(b); end );
    return all;
end;

